`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date: 12/11/2023 09:54:43 PM
// Design Name: 
// Module Name: control_unit
// Project Name: 
// Target Devices: 
// Tool Versions: 
// Description: 
// 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////


module control_unit(
         clk,
         rst_n,
         ir,
         P,
         A,
         Mw,
         Mr,
         LAlt_AC,
         LAlt_MBR,
         debug_microop
    );
    
    parameter word_width  = 16;
    parameter state_width = 8;
    
    input                    clk;
    input                    rst_n;
    input [word_width-1 : 0] ir;
    output reg [5:0]         P;
    output reg [2:0]         A;
    output reg               Mw;
    output reg               Mr;
    output reg               LAlt_AC;
    output reg               LAlt_MBR;
    output reg [7:0]         debug_microop;
    
    //define states
    `define reset                   'h00            /* reset state */
    `define fetch                   'h10            /* load instruction to instruction register */
    `define decode                  'h20            /* analyze loaded instruction */
    `define get_op                  'h30            /* load operand from register */
    `define exec_load               'h40            /* exec the load*/  
    `define exec_alu                'h41            /* exec the arithnmetical op*/ 
    
    //define commands
    `define ir_we                   3'd7
    `define ir_oe                   3'd7
    `define out_we                  3'd6
    `define out_oe                  3'd6
    `define in_we                   3'd5
    `define in_oe                   3'd5
    `define ac_we                   3'd4
    `define ac_oe                   3'd4
    `define mbr_we                  3'd3
    `define mbr_oe                  3'd3
    `define pc_we                   3'd2
    `define pc_oe                   3'd2
    `define mar_we                  3'd1
    `define mar_oe                  3'd1
    `define regs_nop                3'd0
    
    
    reg [state_width-1 : 0] state = `reset, state_next;
        
    // FSM - sequential part
    always @(posedge clk) begin
        state <= `reset;
    
        if(rst_n)
            state <= state_next;
    end
    
    always @(*) begin
        state_next    = `reset;
        debug_microop = 0;
        P       = 0;
        Mw      = 0;
        Mr      = 0;
        A       = 0;
        LAlt_AC = 0;
        LAlt_MBR= 0;
        
        case(state)
            `reset: begin
                state_next = `fetch;
            end

            `fetch: begin
                P = {`mar_we, `pc_oe};
                
                debug_microop[0] = 1;
                state_next = `fetch + 1;
            end

            `fetch + 'd1: begin
                P = {`mbr_we, `regs_nop};
                Mr= 1;
                
                debug_microop[1] = 1;
                state_next = `fetch + 2;
            end

            `fetch + 'd2: begin
                P = {`ir_we, `mbr_oe};
                
                debug_microop[2] = 1;
                state_next = `fetch + 'd3;
            end
            
            `fetch + 'd3: begin
                P = {`pc_we, `pc_oe};
                A[2] = 1;
                
                debug_microop[3] = 1;
                state_next = `decode;
            end   
            
            `decode: begin
                P = {`mar_we, `ir_oe};
                
                debug_microop[4] = 1;
                state_next = `get_op;
            end  
            
            `get_op: begin
                P = {`mbr_we, `regs_nop};
                Mr= 1;
                
                debug_microop[5] = 1;
                state_next = ir[15:12] == 4'b0001 ? `exec_load : `exec_alu;
            end    
            
            `exec_load: begin
                P = {`ac_we, `mbr_oe};
                
                debug_microop[6] = 1;
                state_next = `fetch;
            end
            
            `exec_alu: begin
                P = {`ac_we, `regs_nop};
                LAlt_AC = 1;
                A[1:0] = ir[15:12] == 4'b0011 ? 1 : ir[15:12] == 4'b0100 ? 2 : ir[15:12] == 4'b1010 ? 3 : 0;
                
                debug_microop[6] = 1;
                state_next = `fetch;
            end
        endcase
            
    end
endmodule
